"""Create Manim Editor project."""
import json
import os
from pathlib import Path
from typing import List, Tuple

from .manim_loader import get_scenes
from .presentation_classes import Section, Slide


def create_project_dir(project_name: str) -> Tuple[bool, str]:
    """Ensure existence of project dir.
    Return True if success and message for the user.
    """
    print(f"Creating project '{project_name}'.")
    # TODO: valid dir name check for Windows
    if project_name == "":
        return False, "The project name can't be empty."
    if os.path.exists(project_name):
        # empty dirs are ok
        if len(os.listdir(project_name)) == 0:
            return True, f"The project directory '{project_name}' has already been created. It will be used."
        return (
            False,
            f"The project name '{project_name}' points to a filled directory. If this is a project, you can open it instead.",
        )
    try:
        os.mkdir(project_name)
    except FileNotFoundError:
        return False, f"Creation of project directory '{project_name}' failed."
    return True, f"Created new project directory '{project_name}'."


def populate_project_with_loaded_sections(project_name: str, sections: List[Section]) -> bool:
    if not len(sections):
        raise RuntimeError(f"No sections given for project '{project_name}'.")
    if sections[0].is_sub_section:
        raise RuntimeError(f"The first section of project '{project_name}' can't be a sub section.")

    # load slides
    slides: List[Slide] = []
    for id, section in enumerate(sections):
        # add sub section
        if section.is_sub_section:
            if not slides[-1].populate_sub_section(section):
                return False
        else:
            # set main section
            slides.append(Slide())
            if not slides[-1].populate_main_section(section, project_name, id):
                return False

    # write project file
    with open(Path(project_name) / "project.json", "w") as file:
        json.dump([slide.get_dict() for slide in slides], file, indent=4)
    return True


def populate_project(project_name: str, scene_ids: List[int]) -> bool:
    """Create project JSON file, copy selected section video files and create thumbnails.
    Return False at failure.
    """
    print(f"Populating project '{project_name}'.")
    # select scenes according to ids set by frontend
    scenes = [scene for scene in get_scenes() if scene.id in scene_ids]
    sections: List[Section] = []
    for scene in scenes:
        sections += scene.sections

    success = populate_project_with_loaded_sections(project_name, sections)
    if success:
        # xD
        print(f"Project '{project_name}' population finished.")
    else:
        print(f"Project '{project_name}' population failed.")
    return success



from .assets import *
from .commands import *
from .config import *
from .create_project import *
from .edit_project import *
from .load_project import *
from .manim_loader import *
from .presentation_classes import *



import json
import os
import subprocess
from pathlib import Path
from typing import Any, Generator, List, Optional, Tuple

import jsonschema

from .config import get_config


def capture(command: List[str], cwd: Optional[Path] = None, command_input: Optional[str] = None) -> Tuple[str, str, int]:
    """Run command and capture standard output, standard error and return code."""
    p = subprocess.run(command, cwd=cwd, input=command_input, capture_output=True, text=True)
    out, err = p.stdout, p.stderr
    return out, err, p.returncode


def capture_ffmpeg(params: List[str]) -> Tuple[str, str, int]:
    """Run ffmpeg with params and capture standard output, standard error and return code."""
    return capture(["ffmpeg"] + params)


def run_ffmpeg(params: List[str]) -> bool:
    """Run ffmpeg with params and loglevel defined in config, print output to terminal and return True at success."""
    return subprocess.call(["ffmpeg", "-loglevel", get_config().FFMPEG_LOGLEVEL] + params) == 0


def walk(top: Path, maxdepth: int) -> Generator[Tuple[Path, List[str], List[str]], None, None]:
    """Reimplementation of os.walk with max recursion depth."""
    dirs: List[str] = []
    nondirs: List[str] = []
    for name in os.listdir(top):
        if os.path.isdir(top / name):
            dirs.append(name)
        else:
            nondirs.append(name)
    yield top, dirs, nondirs
    if maxdepth:
        for name in dirs:
            for res in walk(top / name, maxdepth - 1):
                yield res


def valid_json_load(path: Path, schema: Any) -> Optional[Any]:
    """Load json file from path and validate with schema.
    Return ``None`` when json is invalid.
    """
    if not os.path.isfile(path):
        return None
    with open(path, "r") as file:
        try:
            data = json.load(file)
        except json.decoder.JSONDecodeError:
            return None
    try:
        jsonschema.validate(data, schema)
    except jsonschema.ValidationError:
        return None
    return data



from .config import Config
from .editor import (
    EditorBanner,
    EditorLogo,
    IconCompleteLoop,
    IconLoop,
    IconNormal,
    IconSkip,
    PresentationSectionType,
    get_config,
    set_config,
)

set_config(Config)


# NORMAL = PresentationSectionType.NORMAL
# SKIP = PresentationSectionType.SKIP
# LOOP = PresentationSectionType.LOOP
# COMPLETE_LOOP = PresentationSectionType.COMPLETE_LOOP



from flask import Flask


def create_app(config) -> Flask:
    app = Flask(__name__)
    app.config.from_object(config)

    # blueprints
    from .error import bp as errors_bp

    app.register_blueprint(errors_bp)

    from .main import bp as main_bp

    app.register_blueprint(main_bp)

    return app



from distutils.dir_util import copy_tree
from pathlib import Path
from typing import List

import jinja2

from .config import get_config
from .presentation_classes import Slide


def emulate_url_for(endpoint: str, path: str = "", name: str = "", filename: str = "") -> str:
    """Translate flask urls into local urls for project.
    Can't be ``Path`` since flask uses strings here.
    ``name`` is being ignored as the output path will be from the project dir already.
    """
    if endpoint == "static":
        return filename
    elif endpoint == "main.serve_project_static":
        return path
    raise ValueError(f"Endpoint '{endpoint}' can't be emulated.")


def export_presentation(project_name: str, slides: List[Slide]) -> None:
    print(f"Exporting Project '{project_name}' as presentation.")
    jinja2_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(
            [
                get_config().ROOT_DIR / "app" / "templates",
                get_config().ROOT_DIR / "app" / "main" / "templates",
                get_config().ROOT_DIR / "app" / "error" / "templates",
            ]
        ),
        autoescape=jinja2.select_autoescape(["html"]),
    )

    html = jinja2_env.get_template("edit_project.html").render(
        present_export=True, version=get_config().VERSION, name=project_name, slides=slides, url_for=emulate_url_for
    )
    with open(Path(project_name) / "index.html", "w") as file:
        file.write(html)
    copy_tree(get_config().ROOT_DIR / "app" / "static" / "webpack", str(Path(project_name) / "webpack"))
    copy_tree(get_config().ROOT_DIR / "app" / "static" / "img", str(Path(project_name) / "img"))
    print(f"Presentation is ready at '{Path(project_name).absolute()}'")
    print("Run 'python3 -m http.server' in that directory. Simply opening the .html file won't work.")



"""Load a Manim Editor project."""
from pathlib import Path
from typing import Dict, List, Optional, Tuple

from .commands import walk
from .config import get_config
from .manim_loader import valid_json_load
from .presentation_classes import Slide


def get_project(name: str) -> Tuple[Optional[str], List[Slide]]:
    """Parse project json file if valid.
    ``path`` is path to prject dir, not to the json index file.
    Otherwise return ``None`` as name.
    """
    path = Path(name)
    raw_slides = valid_json_load(path / "project.json", get_config().PROJECT_SCHEMA)
    if raw_slides is None:
        return None, []
    slides: List[Slide] = []
    for raw_slide in raw_slides:
        slide = Slide()
        slide.load(raw_slide)
        slides.append(slide)
    return name, slides


def get_projects() -> Dict[str, List[Slide]]:
    """Get all projects in the current working directory.
    No recursive search will be applied."""
    # get all projects in this dir
    project_paths: List[Path] = []
    for root, _, files in walk(Path("."), 1):
        for file in files:
            if file == "project.json":
                project_paths.append(root)
    projects: Dict[str, List[Slide]] = {}
    for project_name in project_paths:
        name, slides = get_project(str(project_name))
        # don't include invalid
        if name is not None:
            projects[name] = slides
    return projects



import json
import os
from pathlib import Path
from typing import Any

BASE_DIR = Path(__file__).parent.absolute()


class Config:
    # NOTE: don't use this in an online environment, it invalidates all sessions when restarting
    SECRET_KEY = os.urandom(16)
    # one of: quiet, panic, fatal, error, warning, info, verbose, debug, trace
    FFMPEG_LOGLEVEL = "error"
    # protect user from scanning entire system when run in root
    RECURSION_DEPTH = 10

    ROOT_DIR = BASE_DIR
    STATIC_DIR = BASE_DIR / "app" / "static"

    VERSION = "v0.3.0"

    with open(BASE_DIR / "section_index.schema.json", "r") as file:
        SECTION_INDEX_SCHEMA = json.load(file)
    with open(BASE_DIR / "project.schema.json", "r") as file:
        PROJECT_SCHEMA = json.load(file)

    @staticmethod
    def set(key: str, value: Any):
        if getattr(Config, key) is None:
            raise ValueError(f"Key '{key}' doesn't exist in config")
        setattr(Config, key, value)



import os
import time
from enum import Enum
from fractions import Fraction
from pathlib import Path
from typing import Any, Dict, List

from .commands import run_ffmpeg


class PresentationSectionType(str, Enum):
    # start, end, wait for continuation by user
    NORMAL = "presentation.normal"
    # start, end, immediately continue to next section
    SKIP = "presentation.skip"
    # start, end, restart, immediately continue to next section when continued by user
    LOOP = "presentation.loop"
    # start, end, restart, finish animation first when user continues
    COMPLETE_LOOP = "presentation.complete_loop"

    # same as above but define sub section
    SUB_NORMAL = "presentation.sub.normal"
    SUB_SKIP = "presentation.sub.skip"
    SUB_LOOP = "presentation.sub.loop"
    SUB_COMPLETE_LOOP = "presentation.sub.complete_loop"


class Slide:
    """An instance of this class is represented as one element in the timeline of the presenter.
    It can hold one full section or one full section and multiple sub sections.
    A presentation consists of multiple slides.

    Attributes
    ----------
    sections
        List of sections in this slide
    """

    def __init__(self):
        self.sections: List[Section] = []

    def populate_main_section(self, main_section: "Section", project_name: str, in_project_id: int) -> bool:
        """Load first section."""
        if not main_section.set_project(project_name, in_project_id, in_project_id):
            return False
        self.sections = [main_section]
        return True

    def populate_sub_section(self, sub_section: "Section") -> bool:
        """meth:`populate` must have been executed first."""
        if not sub_section.set_project(
            self.sections[0].project_name, self.sections[-1].in_project_id + 1, self.sections[0].in_project_id
        ):
            return False
        self.sections.append(sub_section)
        return True

    def get_dict(self) -> Dict[str, Any]:
        """Get dictionary representation."""
        return {
            "sections_len": len(self.sections),
            "sections": [section.get_dict() for section in self.sections],
        }

    def __load_section(self, raw_section: Dict[str, Any]) -> "Section":
        """Create :class:`.Section` from dict read from a project json file created by the Manim Editor.
        Assumes the input data to be valid.
        """

        return Section(
            int(raw_section["id"]),
            raw_section["name"],
            raw_section["type"],
            raw_section["is_sub_section"],
            raw_section["original_video"],
            int(raw_section["width"]),
            int(raw_section["height"]),
            Fraction(raw_section["fps"]),
            float(raw_section["duration"]),
            # this hasn't been created by Manim directly
            raw_section["project_name"],
            raw_section["in_project_video"],
            raw_section["in_project_thumbnail"],
            int(raw_section["in_project_id"]),
        )

    def load(self, raw_slide: Dict[str, Any]) -> None:
        """Create :class:`.Slide` from dict read from a project json file created by the Manim Editor.
        Assumes the input data to be valid.
        """
        for section in raw_slide["sections"]:
            self.sections.append(self.__load_section(section))


class Section:
    """Representation of Manim :class:`.Section`.
    It can either be a full section of a sub section.

    Attributes
    ----------
    id
        Unique id for this section.
    name
        Human readable, non-unique name for this section.
    type
        How should this section be played?
    is_sub_section
        ``True`` when this is a subsection rather than a section.
    original_video
        Path to original video file.
    width
        width of the video
    height
        Height of the video.
    fps
        Frame rate of the video as :class:`.Fraction`.
    duration
        Duration of the video in seconds.
    project_name
        Name of the project this section is used in.
    in_project_video
        Path to copied video file relative to project file.
    in_project_thumbnail
        Path to thumbnail file relative to project file.
    in_project_id
        Id for this section that is unique in its project.
    parent_id
        In project id of the section this sub-section belongs to.
        When this is not a sub section it holds its own id.
        This is similar to a union-find structure.

    See Also
    --------
    :class:`.PresentationSectionType`
    """

    def __init__(
        self,
        id: int,
        name: str,
        type: str,
        is_sub_section: bool,
        original_video: Path,
        width: int,
        height: int,
        fps: Fraction,
        duration: float,
        # only to be used when loading from project file
        project_name: str = "",
        in_project_video: Path = Path(),
        in_project_thumbnail: Path = Path(),
        in_project_id: int = -1,
        parent_id: int = -1,
    ):
        self.id = id
        self.name = name
        self.type = type
        self.is_sub_section = is_sub_section
        self.original_video = original_video
        self.width = width
        self.height = height
        self.fps = fps
        self.duration = duration
        # to be set once project is being populated
        self.project_name = project_name
        self.in_project_video = in_project_video
        self.in_project_thumbnail = in_project_thumbnail
        self.in_project_id = in_project_id
        self.parent_id = parent_id

    def get_in_project_video_abs(self) -> Path:
        return Path(self.project_name) / self.in_project_video

    def get_in_project_thumbnail_abs(self) -> Path:
        return Path(self.project_name) / self.in_project_thumbnail

    def set_project(self, project_name: str, in_project_id: int, parent_id: int) -> bool:
        """Hand this video over to a project.
        Update attributes and copy assets.
        """
        self.project_name = project_name
        self.in_project_id = in_project_id
        self.in_project_video = Path(f"video_{in_project_id:04}.mp4")
        self.in_project_thumbnail = Path(f"thumb_{in_project_id:04}.jpg")
        self.parent_id = parent_id

        return self.convert_video() and self.extract_thumbnail()

    def convert_video(self) -> bool:
        """Convert original video into fragmented format that can be played by the video player.
        According to https://developer.mozilla.org/en-US/docs/Web/API/Media_Source_Extensions_API/Transcoding_assets_for_MSE.
        Also copy it to project dir.
        Return False at failure.
        """
        # TODO: this doesn't work for some versions of Chromium
        print(f"Converting video to '{self.in_project_video}' for section '{self.name}'.")
        if not run_ffmpeg(
            [
                "-i",
                str(self.original_video),
                "-movflags",
                "frag_keyframe+empty_moov+default_base_moof",
                str(self.get_in_project_video_abs()),
                "-y",
            ]
        ):
            print("Video Conversion failed.")
            return False
        return True

    def extract_thumbnail(self) -> bool:
        """Create thumbnail for section in project dir.
        Return False at failure.
        """
        print(f"Extracting thumbnail '{self.in_project_thumbnail}' from video for section '{self.name}'.")
        if not run_ffmpeg(
            [
                "-sseof",
                "-3",
                "-i",
                str(self.original_video),
                "-update",
                "1",
                "-q:v",
                "1",
                str(self.get_in_project_thumbnail_abs()),
                "-y",
            ]
        ):
            print("Thumbnail extraction failed.")
            return False
        return True

    def get_dict(self) -> Dict[str, Any]:
        """Get dictionary representation."""
        return {
            "id": self.id,
            "name": self.name,
            "type": self.type,
            "is_sub_section": self.is_sub_section,
            "original_video": str(self.original_video),
            "width": self.width,
            "height": self.height,
            "fps": str(self.fps),
            "duration": self.duration,
            "project_name": self.project_name,
            "in_project_video": str(self.in_project_video),
            "in_project_thumbnail": str(self.in_project_thumbnail),
            "in_project_id": self.in_project_id,
            "parent_id": self.parent_id,
        }


class Scene:
    """Representation of the entire section index of one scene.

    Attributes
    ----------
    id
        unique id for this scene
    name
        name for the represented scene
    path
        path to index JSON file
    last_modified
        seconds since the epoch of last modification of index file
    sections
        list of sections in scene
    """

    def __init__(self, id: int, name: str, path: Path, last_modified: float, sections: List[Section]):
        self.id = id
        self.name = name
        self.path = path
        self.last_modified = last_modified
        self.sections = sections

    def get_last_modified(self) -> str:
        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.last_modified))

    def get_rel_dir_path(self) -> Path:
        """Used to display location in frontend."""
        parent_path = self.path.parent.absolute()
        return parent_path.relative_to(os.getcwd())



import manim

from .config import get_config


class IconNormal(manim.VGroup):
    def __init__(self):
        manim.VGroup.__init__(self)
        icon_normal = manim.SVGMobject(str(get_config().STATIC_DIR / "img" / "play_btn.svg"))
        self.add(icon_normal)


class IconSkip(manim.VGroup):
    def __init__(self):
        manim.VGroup.__init__(self)
        icon_skip = manim.SVGMobject(str(get_config().STATIC_DIR / "img" / "wind.svg"))
        self.add(icon_skip)


class IconLoop(manim.VGroup):
    def __init__(self):
        manim.VGroup.__init__(self)
        icon_loop = manim.SVGMobject(str(get_config().STATIC_DIR / "img" / "arrow_clockwise.svg"))
        self.add(icon_loop)


class IconCompleteLoop(manim.VGroup):
    def __init__(self):
        manim.VGroup.__init__(self)
        icon_completeloop = manim.SVGMobject(str(get_config().STATIC_DIR / "img" / "hourglass_split.svg"))
        self.add(icon_completeloop)


class EditorLogo(manim.VGroup):
    def __init__(self):
        manim.VGroup.__init__(self)
        logo_green = "#87c2a5"
        logo_blue = "#525893"
        logo_red = "#e07a5f"
        triangle = manim.Triangle(color=logo_red, fill_opacity=1, z_index=10).shift(1.2 * manim.RIGHT)
        square = manim.Square(color=logo_blue, fill_opacity=1, z_index=20).shift(manim.UP)
        circle = manim.Circle(color=logo_green, fill_opacity=1, z_index=30).shift(manim.LEFT)
        logo = manim.VGroup(triangle, square, circle).scale(2.5)
        logo.move_to(manim.ORIGIN)
        self.add(logo)

        l1 = manim.Rectangle(height=1.7, width=0.6, stroke_width=0, fill_opacity=1, z_index=11)
        l2 = l1.copy()
        pause = manim.VGroup(l1, l2).arrange(manim.RIGHT)
        pause.move_to(triangle.get_center_of_mass())
        self.add(pause)

        play = (
            manim.Triangle(color=manim.WHITE, fill_opacity=1, z_index=21)
            .rotate(-90 * manim.DEGREES)
            .move_to(square.get_center())
            .scale(1.5)
            .shift(0.2 * manim.RIGHT)
        )
        self.add(play)

        inner = 1.2
        outer = 1.75
        arc = manim.Sector(
            arc_center=circle.get_center(),
            inner_radius=inner,
            outer_radius=outer,
            start_angle=-manim.PI / 2,
            angle=-1.5 * manim.PI,
            color=manim.WHITE,
            stroke_width=0,
            z_index=31,
        )
        tip = manim.mobject.geometry.ArrowTriangleFilledTip(z_index=31, color=manim.WHITE, start_angle=-manim.PI / 2).scale(3.5)
        tip.next_to(arc.get_right(), manim.DOWN, buff=0).shift((outer - inner) / 2 * manim.LEFT + 0.05 * manim.UP)

        loop = manim.VGroup(arc, tip).rotate(-manim.PI / 16)
        self.add(loop)


class EditorBanner(manim.VGroup):
    def __init__(self):
        manim.VGroup.__init__(self)

        logo_all = EditorLogo().scale(0.5)

        logo_black = "#343434"
        M = manim.MathTex(r"\mathbb{M}").scale(7).set_color(logo_black)
        anim = (
            manim.Tex(r"\textbf{anim}", tex_template=manim.TexFontTemplates.gnu_freeserif_freesans)
            .scale(7)
            .set_color(logo_black)
            .scale(0.75748)
        )

        E = manim.MathTex(r"\mathbb{E}").scale(7).set_color(logo_black)
        ditor = (
            manim.Tex(r"\textbf{ditor}", tex_template=manim.TexFontTemplates.gnu_freeserif_freesans)
            .scale(7)
            .set_color(logo_black)
            .scale(0.75748)
        )
        x = manim.VGroup(M, anim).arrange(manim.RIGHT, buff=0.1)
        y = manim.VGroup(E, ditor).arrange(manim.RIGHT, buff=0.1)
        banner = manim.VGroup(x, y, logo_all).arrange(manim.RIGHT, buff=0.5)
        anim.align_to(M, manim.DOWN)
        ditor.align_to(M, manim.DOWN)
        self.add(
            manim.RoundedRectangle(
                fill_color="#ece6e2",
                height=banner.get_height() + 1,
                width=banner.get_width() + 1.5,
                fill_opacity=1,
                stroke_width=0,
                z_index=0,
            )
        )
        self.add(banner)
        self.scale(0.3)



import logging
import socket
from pathlib import Path
from typing import List, Optional

import click
from waitress import serve

from .app import create_app
from .config import Config
from .editor import (
    Section,
    create_project_dir,
    export_presentation,
    get_project,
    get_scene,
    populate_project_with_loaded_sections,
    set_config,
)


def find_open_port():
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.bind(("", 0))
    sock.listen(1)
    _, port = sock.getsockname()
    sock.close()

    return port


def run_normal() -> None:
    set_config(Config)
    app = create_app(Config)
    # disable logging for every get/post request
    log = logging.getLogger("werkzeug")
    log.setLevel(logging.ERROR)
    port = find_open_port()
    url = f"http://localhost:{port}"
    print(f"Starting Manim Editor {Config.VERSION}. Open {url} in a browser. (Press CTRL+C to quit)")
    serve(app, host="0.0.0.0", port=port)


def run_debug() -> None:
    Config.set("FFMPEG_LOGLEVEL", "info")
    set_config(Config)
    app = create_app(Config)
    app.run(debug=True)


def run_quick_present_export(section_index_paths: List[Path], project_name: Optional[str] = None) -> None:
    """Create a project from scene(s) and export the presentation.
    Using the name of the first project if ``project_name`` is not given.
    """
    set_config(Config)
    if project_name is None:
        project_name = section_index_paths[0].name[:-5]

    success, msg = create_project_dir(project_name)
    if not success:
        raise RuntimeError(msg)

    sections: List[Section] = []
    for section_index_path in section_index_paths:
        scene = get_scene(section_index_path, 0)
        if scene is None:
            raise RuntimeError(f"Couldn't find a section index file at '{section_index_path}'.")
        sections += scene.sections
    if not populate_project_with_loaded_sections(project_name, sections):
        raise RuntimeError("Failed to populate project.")
    project_name, slides = get_project(project_name)
    if project_name is None:
        raise RuntimeError("Failed to load project.")
    export_presentation(project_name, slides)


@click.command()
@click.option("--debug", is_flag=True, help="Launch in Debug mode.")
@click.option("--version", is_flag=True, help="Print version and exit.")
@click.option(
    "--quick_present_export",
    help="Create a project from a scene and export the presentation. The path to the section index file must be provided as the argument.",
    multiple=True,
)
@click.option(
    "--project_name", help="Use this name for the project being created use the name of the first section when not provided."
)
def main(debug: bool, version: bool, quick_present_export: List[str], project_name: Optional[str]) -> None:
    if version:
        print(f"Manim Editor {Config.VERSION}")
        return
    if len(quick_present_export):
        run_quick_present_export([Path(section_index_path) for section_index_path in quick_present_export], project_name)
        return
    if debug:
        run_debug()
    else:
        run_normal()


if __name__ == "__main__":
    main()



from flask import render_template

from . import bp


@bp.app_errorhandler(400)
def bad_request(_):
    return render_template("400.html"), 401


@bp.app_errorhandler(401)
def access_denied_error(_):
    return render_template("401.html"), 401


@bp.app_errorhandler(404)
def not_found_error(_):
    return render_template("404.html"), 404


@bp.app_errorhandler(418)
def teapot(_):
    return render_template("418.html"), 418


@bp.app_errorhandler(500)
def internal_error(_):
    return render_template("500.html"), 500



"""Main backend file."""
import os
from pathlib import Path

from flask import (
    abort,
    current_app,
    flash,
    jsonify,
    redirect,
    render_template,
    request,
    send_file,
    url_for,
)

from ...editor import (
    create_project_dir,
    export_presentation,
    get_project,
    get_projects,
    get_scenes,
    populate_project,
)
from . import bp


@bp.route("/")
@bp.route("/index")
def index():
    return redirect(url_for("main.project_selection"))


@bp.route("/project_selection")
def project_selection():
    projects = get_projects()
    return render_template(
        "project_selection.html",
        version=current_app.config["VERSION"],
        title="Project Selection",
        cwd=os.getcwd(),
        projects=projects,
    )


@bp.route("/edit_project/<name>")
def edit_project(name: str):
    project_name, slides = get_project(name)
    if project_name is None:
        abort(404)
    return render_template(
        "edit_project.html",
        version=current_app.config["VERSION"],
        title="Edit Project",
        name=name,
        slides=slides,
        present_export=False,
    )


# ajax
@bp.route("/export_presentation", methods=["POST"])
def export_presenter():
    project = request.json
    if project is None:
        abort(400)
    project_name, slides = get_project(project["name"])
    if project_name is None:
        print(f"Project '{project['name']}' couldn't be found.")
        return jsonify(success=False)
    export_presentation(project_name, slides)
    return jsonify(success=True)


@bp.route("/serve_project_static/<name>/<path>")
def serve_project_static(name: str, path: str):
    """Since the projects are not in the Flask static folder, this function serves the videos and thumbnails instead."""
    abspath = (Path(name) / path).absolute()
    # NOTE: this should never be used in an online environment, for a local one it's fine
    return send_file(abspath)


@bp.route("/create_project")
def create_project():
    return redirect(url_for("main.set_project_name"))


@bp.route("/create_project1")
def set_project_name():
    return render_template("set_project_name.html", version=current_app.config["VERSION"], title="Create New Project")


@bp.route("/create_project2", methods=["GET", "POST"])
def scene_selection():
    # when users play around with the address bar
    if request.method == "GET":
        return redirect(url_for("main.set_project_name"))
    project_name = request.form["project_name"].strip()
    success, message = create_project_dir(project_name)
    if not success:
        flash(message, "danger")
        return redirect(url_for("main.set_project_name"))

    flash(message, "success")
    scenes = get_scenes()
    if len(scenes) == 0:
        flash(
            "No sections were found. You must render you scene with the '--save_sections' flag. Refer to the documentation for more information.",
            "danger",
        )
    return render_template(
        "scene_selection.html",
        version=current_app.config["VERSION"],
        title="Create New Project",
        scenes=scenes,
        project_name=project_name,
    )


# ajax
@bp.route("/create_project3", methods=["POST"])
def confirm_scene_selection():
    project = request.json
    if project is None:
        abort(400)
    project_name: str = project["name"]
    scene_ids = project["scene_ids"]
    success = populate_project(project_name, scene_ids)
    if success:
        flash(f"The project '{project_name}' has successfully been populated.", "success")
        return jsonify(success=True)
    else:
        # the flash will be taken care of by the client
        abort(500)



"""Functions used to load the render results of Manim."""
import os
from fractions import Fraction
from pathlib import Path
from typing import Any, Dict, List, Optional

from .commands import valid_json_load, walk
from .config import get_config
from .presentation_classes import Scene, Section


def get_manim_section(raw_section: Dict[str, Any], index_path: Path, new_id: int) -> Section:
    """Create :class:`.Section` from dict read from a section index created by Manim."""

    parent_path = index_path.parent.absolute()
    type = raw_section["type"].split(".")[-1]
    is_sub_section = raw_section["type"].split(".")[-2] == "sub"
    return Section(
        new_id,
        raw_section["name"],
        type,
        is_sub_section,
        parent_path / raw_section["video"],
        int(raw_section["width"]),
        int(raw_section["height"]),
        Fraction(raw_section["avg_frame_rate"]),
        float(raw_section["duration"]),
    )


def get_scene(path: Path, new_id: int) -> Optional[Scene]:
    """Check if path points to a valid section index file.
    Return ``None`` if file isn't a section index.
    """
    raw_sections = valid_json_load(path, get_config().SECTION_INDEX_SCHEMA)
    if raw_sections is None:
        return None

    sections: List[Section] = []
    for raw_section in raw_sections:
        sections.append(get_manim_section(raw_section, path, len(sections)))
    return Scene(new_id, path.name[:-5], path.absolute(), os.path.getmtime(path), sections)


def get_scenes() -> List[Scene]:
    """Search recursively in CWD for any valid JSON section index files."""
    scene_index_paths: List[Path] = []
    for root, _, files in walk(Path("."), get_config().RECURSION_DEPTH):
        for file in files:
            if file.endswith(".json"):
                scene_index_paths.append(root / file)

    scenes: List[Scene] = []
    for scene_index_path in scene_index_paths:
        scene = get_scene(scene_index_path, len(scenes))
        # valid scene with at least one section?
        if scene is not None and len(scene.sections):
            scenes.append(scene)
    return scenes



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['manim_editor',
 'manim_editor.app',
 'manim_editor.app.error',
 'manim_editor.app.main',
 'manim_editor.editor']

package_data = \
{'': ['*'],
 'manim_editor.app': ['static/img/*', 'static/webpack/*', 'templates/*'],
 'manim_editor.app.error': ['templates/*'],
 'manim_editor.app.main': ['templates/*']}

install_requires = \
['Flask>=2.0.2,<3.0.0',
 'click>=8.0.3,<9.0.0',
 'jsonschema>=4.1.2,<5.0.0',
 'manim>=0.12.0,<0.13.0',
 'waitress>=2.0.0,<3.0.0']

entry_points = \
{'console_scripts': ['manedit = manim_editor.__main__:main',
                     'manim_editor = manim_editor.__main__:main']}

setup_kwargs = {
    'name': 'manim-editor',
    'version': '0.3.0',
    'description': 'Editor and Presenter for Manim Generated Content',
    'long_description': '<p align="center">\n    <a href="#"><img src="https://raw.githubusercontent.com/ManimEditorProject/manim_editor/main/manim_editor/app/static/img/banner.png"></a>\n    <br />\n    <br />\n    <a href="https://pypi.org/project/manim-editor/"><img src="https://img.shields.io/pypi/v/manim-editor.svg?style=flat&logo=pypi" alt="PyPI Latest Release"></a>\n    <a href="https://manim-editor.readthedocs.io/en/stable/"><img src=\'https://readthedocs.org/projects/manim-editor/badge/?version=stable\' alt=\'Documentation Status\' /></a>\n    <a href="http://choosealicense.com/licenses/mit/"><img src="https://img.shields.io/badge/license-MIT-red.svg?style=flat" alt="MIT License"></a>\n    <a href="https://github.com/ManimEditorProject/manim_editor/actions/workflows/build_pages.yml"><img src="https://github.com/ManimEditorProject/manim_editor/actions/workflows/build_pages.yml/badge.svg" alt="MIT License"></a>\n    <br />\n    <br />\n    <i>Editor and Presenter for Manim Generated Content.</i>\n</p>\n<hr />\n\nTake a look at the [Working Example](https://manimeditorproject.github.io/manim_editor/).\nMore information can be found on the [documentation](https://manim-editor.readthedocs.io/en/stable/).\n\nThese Browsers are supported:\n- Firefox\n- Chrome\n- Edge\n\n## Create a Project\n\n![create_project](https://raw.githubusercontent.com/ManimEditorProject/manim_editor/main/docs/source/_static/create_project.gif)\n\n## Export a Project as a Presentation\n\n![export_presenter](https://raw.githubusercontent.com/ManimEditorProject/manim_editor/main/docs/source/_static/export_presenter.gif)\n',
    'author': 'christopher-besch',
    'author_email': 'christopher.besch@gmx.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ManimEditorProject/manim_editor',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
